import os

from flask import Flask, render_template, url_for, jsonify, request, send_from_directory
from flask_socketio import SocketIO, emit
from flask.ext.bower import Bower

from .triggers import triggers_as_dict
from .finders import find_static_asset_dirname

app = Flask(__name__)
app.config.from_object('counter_caller.default_settings')
app.config.from_pyfile(os.path.join(os.getcwd(), 'settings.py'), silent=True)
app.config.from_envvar('COUNTER_CALLER_SETTINGS', silent=True)

sock  = SocketIO(app)
bower = Bower(app)

triggers_dict = triggers_as_dict(app.config['TRIGGERS'])
trigger_ids = map(lambda t: t['id'], triggers_dict)

theme_path = app.config['THEME_PATH']

# If the specified theme path is relative, make it absolute, relative to the cwd.
if not os.path.isabs(theme_path):
    theme_path = os.path.join(os.getcwd(), theme_path)

static_asset_paths = [
    theme_path,
    os.path.join(os.path.dirname(__file__), 'static'),
]

def triggers():
    return triggers_dict

@app.route('/static/<path>')
def find_static(path):
    dirname = find_static_asset_dirname(static_asset_paths, path)

    return send_from_directory(dirname, path)

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/display')
def display():
    return render_template('display.html')

@app.route('/help')
def help():
    return render_template('help.html', triggers=triggers(), aliases=app.config['TRIGGER_ALIASES'])

@app.route('/config.json')
def config():
    return jsonify(triggers=triggers(), please_wait=app.config['PLEASE_WAIT'])

@app.route('/triggers/<trigger_id>', methods=['POST'])
def action_trigger(trigger_id):
    if trigger_id not in trigger_ids:
        # The trigger id might be an alias - try to resolve it
        if trigger_id in app.config['TRIGGER_ALIASES']:
            trigger_id = app.config['TRIGGER_ALIASES'][trigger_id]

    sock.emit('queue', trigger_id, broadcast=True)
    return ('', 204)


@app.route('/actions/refresh', methods=['POST'])
def action_refresh():
    sock.emit('refresh', broadcast=True)
    return ('', 204)

@app.route('/actions/clear', methods=['POST'])
def action_clear():
    sock.emit('clear', broadcast=True)
    return ('', 204)

@sock.on('queue')
def queue(trigger_id):
    action_trigger(trigger_id)

@sock.on('refresh')
def refresh():
    action_refresh()

@sock.on('clear')
def clear():
    action_clear()



from collections import namedtuple

Trigger = namedtuple('Trigger', ['id', 'label', 'shortcut'])

def build_triggers(definitions):
    """
    Convert a series of trigger definitions into trigger objects.

    id:         Can be most anything, but must be unique
    label:      A label to describe the trigger - keep it short
    shortcut:   String that matches the mousetrap.js spec (or an array of same)
    """

    for definition in definitions:
        if type(definition) is str:
            trigger = Trigger(definition, definition, definition)
        elif type(definition) is dict:
            trigger = Trigger(str(definition['id']), str(definition['label']), str(definition['shortcut']))
        else:
            raise RuntimeError('Trigger must be of type str or dict, not %s' % type(definition))

        yield trigger

def triggers_as_dict(definitions):
    triggers = build_triggers(definitions)

    return [ dict(trigger._asdict()) for trigger in triggers ]



import os

def find_static_asset_dirname(search_paths, asset):
    for dirname in search_paths:
        path = os.path.join(dirname, asset)

        if os.path.exists(path):
            return dirname

    return ''



from setuptools import setup, find_packages

VERSION = '2.0.3rc6'
DEPENDS = [
    'Flask==0.10.1',
    'Flask-Bower',
    'Flask-SocketIO',
    'eventlet',
    'click',
]

setup(name='counter-caller', version=VERSION,
      packages=find_packages(), install_requires=DEPENDS,
      author='Lewis Eason', author_email='me@lewiseason.co.uk',
      include_package_data=True,
      package_data={
        'counter-caller': ['*.html', '*.css', '*.js', '*.mp3', '*.jpg']
      },
      entry_points={
        'console_scripts': ['counter-caller = counter_caller.cli:run']
      }
)



import click
from counter_caller.app import app, sock

@click.command()
@click.option('bind', '-b', '--bind', default='0.0.0.0:5000', type=str, help='Host/port to bind to (default: 0.0.0.0:5000)')
def run(bind):
    host, port = bind.split(':', 2)
    sock.run(app, host=host, port=int(port))
